// 003 u_c_field_value_list
// 22 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- maintain the values of simple fields and blob fields

(*$r+*)

unit u_c_field_value_list;
  interface
    uses Classes, DB, u_c_basic_object;

    type c_field_value= // one "field_value"
                        Class(c_basic_object)
                          // -- m_name: the field_value name
                          m_field_type: tFieldType;

                          // -- filled at parse time
                          m_field_index: Integer;
                          m_value: String;
                          m_c_stream: tStream;

                          Constructor create_field_value(p_name: String;
                            p_field_type: tFieldType;
                            p_value: String; p_c_stream: tStream);
                          function f_display_field_value: String;
                          function f_display_field_value_structure: String;
                          function f_c_self: c_field_value;
                          Destructor Destroy; Override;
                        end; // c_field_value

         c_field_value_list= // "field_value" list
                             Class(c_basic_object)
                               m_c_field_value_list: tStringList;

                               Constructor create_field_value_list(p_name: String);

                               function f_field_value_count: Integer;
                               function f_c_field_value(p_field_value_index: Integer): c_field_value;
                               function f_index_of(p_field_value_name: String): Integer;
                               function f_c_find_by_field_value(p_field_value_name: String): c_field_value;
                               procedure add_field_value(p_field_value_name: String; p_c_field_value: c_field_value);
                               function f_c_add_field_value(p_field_value_name: String;
                                   p_field_type: tFieldType; p_value: String;
                                   p_c_stream: tStream): c_field_value;
                               function f_c_add_unique_field_value(p_field_value_name: String;
                                   p_field_type: tFieldType; p_value: String;
                                   p_c_stream: tStream): c_field_value;

                               procedure display_field_value_list;
                               procedure display_field_value_list_structure;

                               Destructor Destroy; Override;
                             end; // c_field_value_list

  implementation
    uses SysUtils, u_c_display, u_db;

    // -- c_field_value

    Constructor c_field_value.create_field_value(p_name: String;
        p_field_type: tFieldType;
        p_value: String; p_c_stream: tStream);
      begin
        Inherited create_basic_object(p_name);

        m_field_type:= p_field_type;
        m_value:= p_value;
        m_c_stream:= p_c_stream;

        // -- at creation, by default do not include the field
        m_field_index:= -1;
      end; // create_field_value

    function c_field_value.f_display_field_value: String;
      begin
        Result:= Format('%-20s %3d', [m_name, m_field_index]);
      end; // f_display_field_value

    function c_field_value.f_display_field_value_structure: String;
      begin
        Result:= Format('%-20s %-10s %3d', [m_name, f_fieldtype_name(m_field_type), m_field_index]);
        if m_c_stream= Nil
          then
          else Result:= Result+ ' [blob]';
      end; // f_display_field_value_structure

    function c_field_value.f_c_self: c_field_value;
      begin
        Result:= Self;
      end; // f_c_self

    Destructor c_field_value.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_field_value_list

    Constructor c_field_value_list.create_field_value_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_field_value_list:= tStringList.Create;
      end; // create_field_value_line

    function c_field_value_list.f_field_value_count: Integer;
      begin
        Result:= m_c_field_value_list.Count;
      end; // f_field_value_count

    function c_field_value_list.f_c_field_value(p_field_value_index: Integer): c_field_value;
      begin
        Result:= c_field_value(m_c_field_value_list.Objects[p_field_value_index]);
      end; //  f_c_field_value

    function c_field_value_list.f_index_of(p_field_value_name: String): Integer;
      begin
        Result:= m_c_field_value_list.IndexOf(p_field_value_name);
      end; // f_index_of

    function c_field_value_list.f_c_find_by_field_value(p_field_value_name: String): c_field_value;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_field_value_name);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_field_value(m_c_field_value_list.Objects[l_index_of]);
      end; // f_c_find_by_name

    procedure c_field_value_list.add_field_value(p_field_value_name: String; p_c_field_value: c_field_value);
      begin
        m_c_field_value_list.AddObject(p_field_value_name, p_c_field_value);
      end; // add_field_value

    function c_field_value_list.f_c_add_field_value(p_field_value_name: String;
        p_field_type: tFieldType; p_value: String;
        p_c_stream: tStream): c_field_value;
      begin
        Result:= c_field_value.create_field_value(p_field_value_name,
            p_field_type, p_value, p_c_stream);
        add_field_value(p_field_value_name, Result);
      end; // f_c_add_field_value

    function c_field_value_list.f_c_add_unique_field_value(p_field_value_name: String;
        p_field_type: tFieldType; p_value: String;
        p_c_stream: tStream): c_field_value;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_field_value_name);
        if l_index_of>= 0
          then display_bug_halt('dup_field_value' + p_field_value_name)
          else Result:= f_c_add_field_value(p_field_value_name, p_field_type, p_value, p_c_stream);
      end; // f_c_add_unique_field_value

    procedure c_field_value_list.display_field_value_list;
      var l_field_value_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_field_value_count));

        for l_field_value_index:= 0 to f_field_value_count- 1 do
          display(f_c_field_value(l_field_value_index).f_display_field_value);
      end; // display_field_value_list

    procedure c_field_value_list.display_field_value_list_structure;
      var l_field_value_index: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_field_value_count));

        for l_field_value_index:= 0 to f_field_value_count- 1 do
          display(f_c_field_value(l_field_value_index).f_display_field_value_structure);
      end; // display_field_value_list_structure

    Destructor c_field_value_list.Destroy;
      var l_field_value_index: Integer;
      begin
        for l_field_value_index:= 0 to f_field_value_count- 1 do
          f_c_field_value(l_field_value_index).Free;
        m_c_field_value_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_field_value_list
    end. // u_c_field_value_list
