// 003 u_c_insert_ib_blob
// 22 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

// -- receives a INSERT INTO request, and execute it
// --   mainly created to insert blobs using tStreams

// -- todo
// --  handle empty fields "INSERT INTO  xxx, , yyy, ... "

unit u_c_insert_ib_blob;
  interface
    uses Classes
        , IbDatabase
        , u_c_basic_object
        , u_types_constants
        , u_c_field_value_list
        ;

    type c_insert_ib_blob= class(c_basic_object)
                             m_c_ibdatabase_ref: tIbDatabase;

                             m_c_request: tStringList;
                             m_c_field_value_list: c_field_value_list;

                             m_trace_script: Boolean;
                             m_exec_sql: Boolean;
                             m_explain: String;
                             m_request_count_max: Integer;
                             m_table_count_max: Integer;

                             m_c_table_name_list: tStringList;

                             Constructor create_insert_ib_blob(p_name: String;
                                 p_c_ibdatabase_ref: tIbDatabase);

                               procedure _trace_script(p_text: String);
                               procedure _create_field_value_list(p_table_name: String);
                               procedure _insert_into_blob;
                             procedure execute_script(p_full_script_file_name: String);

                             Destructor Destroy; Override;
                           end; // c_insert_ib_blob

  implementation
    Uses SysUtils
        , Db, IbQuery, IbSql
        , u_c_display
        , u_strings
        , u_ibx, u_db
        , u_c_ib_script
        ;

    // -- c_insert_ib_blob

    Constructor c_insert_ib_blob.create_insert_ib_blob(p_name: String;
        p_c_ibdatabase_ref: tIbDatabase);
      begin
        Inherited create_basic_object(p_name);

        m_c_ibdatabase_ref:= p_c_ibdatabase_ref;

        m_c_request:= tStringList.Create;
      end; // create_insert_ib_blob

    procedure c_insert_ib_blob._trace_script(p_text: String);
      begin
        if m_trace_script
          then display(p_text);
      end; // _trace_script

    procedure c_insert_ib_blob._create_field_value_list(p_table_name: String);
      var l_c_ibquery: tIbQuery;
          l_field_index: Integer;
          l_c_field_value: c_field_value;
      begin
        display('> create_field_value_list');

        l_c_ibQuery:= tIbQuery.Create(Nil);

        with l_c_ibQuery do
        begin
          Database:= m_c_ibdatabase_ref;

          open_ibquery(l_c_ibQuery, 'SELECT * FROM '+ p_table_name);

          for l_field_index:= 0 to FieldCOunt- 1 do
            with FieldDefs[l_field_index] do
            begin
              l_c_field_value:= m_c_field_value_list.f_c_add_field_value(Name, DataType, '', Nil);
              if Name= ''
                then display_bug_stop('no_name');

              // -- create the strem for the blob data, now
              case DataType of
                ftBlob :
                    begin
                      _trace_script('  blob');
                      l_c_field_value.m_c_stream:= tMemoryStream.Create;
                    end;
                ftMemo :
                    begin
                      _trace_script('  memo '+ Name);
                      // stop('memo_field');
                      l_c_field_value.m_c_stream:= tMemoryStream.Create;
                    end;
                // ftGraphic : ;
               end; // case
            end; // for l_field_index, with FieldDefs

          Close;
        end; // with l_c_ibquery

        display('< create_field_value_list');
      end; // _create_field_value_list

    procedure c_insert_ib_blob._insert_into_blob;
        // -- insert the usual field, use script for blob fields
      var l_c_ibsql: tIbSql;
          l_table_name: String;

      procedure create_ibsql_open_database;
        begin
          l_c_ibsql:= tIbSql.Create(Nil);
          with l_c_ibsql do
          begin
            try
              display('exec');
              Database:= m_c_ibdatabase_ref;

              f_open_ib_database('open_ib_db', DataBase);
            except
              on e: Exception do
                begin
                  m_explain:= '*** pb_open_db'+ e.Message;
                  display(m_explain);
                end;
            end; // try except
          end; // with l_c_ibsql
        end; // create_ibsql_open_database

      procedure analyze_request(p_text: String);
        var l_index, l_text_length: Integer;

        function f_extract_simple_field_value(p_field_type: tFieldType): String;
          begin
            case p_field_type of
              ftInteger, ftSmallint, ftBCD :
                  Result:= f_string_extract_characters_in(p_text, l_index, k_digits);
              ftString : Result:= f_string_extract_pascal_string(p_text, l_index);
(*
              ftFloat :
                    begin
                        // -- double
                        _trace_script('  ftFloat');
                        l_values:= l_values+ Fields[l_field_index].AsString;
                      end;
*)
              else
                    display_bug_stop('not_implem '+ f_fieldtype_name(p_field_type));
            end; // f_string_extract_pascal_string(p_text, l_index)
          end; // f_extract_simple_field_value

        var l_insert, l_into: String;

            l_field_name: String;
            l_field_index: Integer;
            l_field_index_of: Integer;

            l_parenthesis, l_comma_parenthesis: Char;
            l_values: String;
            l_string_value: String;

        begin // analyze_request
          display('> analyze_request ');

          l_index:= 1;
          l_text_length:= Length(p_text);
          // display('len '+ IntToStr(l_text_length)+ ' |'+ p_text+ '|');
          // stop('');
          l_insert:= f_string_extract_non_blank(p_text, l_index);
          l_into:= f_string_extract_non_blank(p_text, l_index);
          l_table_name:= f_string_extract_pascal_identifier(p_text, l_index);

          skip_blanks(p_text, l_index);
          l_parenthesis:= f_string_extract_character(p_text, l_index);

          l_field_index:= 0;
          repeat
            l_field_name:= f_string_extract_pascal_identifier(p_text, l_index);
            display('param '+ l_field_name);

            l_field_index_of:= m_c_field_value_list.f_index_of(l_field_name);
            if l_field_index_of>= 0
              then
                with m_c_field_value_list.f_c_field_value(l_field_index_of) do
                begin
                  // -- also add the CASE SENSITIVE name
                  m_name:= l_field_name;
                  m_field_index:= l_field_index;
                end;

            skip_blanks(p_text, l_index);
            l_comma_parenthesis:= f_string_extract_character(p_text, l_index);
            Inc(l_field_index);
          until l_comma_parenthesis= ')';

          l_values:= f_string_extract_pascal_identifier(p_text, l_index);

          skip_blanks(p_text, l_index);
          l_parenthesis:= f_string_extract_character(p_text, l_index);

          l_field_index:= 0;
          // display(p_text+ '|');
          repeat
            with m_c_field_value_list.f_c_field_value(l_field_index) do
              if m_field_type in [ftBlob, ftMemo]
                then begin
                    display('  blob');
                    // -- assumes string blob, with pascal '' escape
                    l_string_value:= f_string_extract_pascal_string(p_text, l_index);
                    // -- remove start and end quotes
                    l_string_value:= f_remove_quote(l_string_value, '''');
                    // display('    blob >'+ l_string_value+ '|');

                    // -- place in a stream
                    with tMemoryStream(m_c_stream) do
                    begin
                      Position:= 0;
                      if Length(l_string_value)> 0
                        then Write(l_string_value[1], Length(l_string_value));
                    end; // with tMemoryStream(m_c_stream)

                    // -- "___,___" or  "__)__"
                  end
                else begin
                    l_string_value:= f_extract_simple_field_value(m_field_type);
                    display('  non_blob '+ l_string_value+ '<');
                    m_value:= l_string_value;
                  end;

            // -- "___,___" or  "__)__"
            skip_blanks(p_text, l_index);
(*
            if l_index<= l_text_length
              then display('  end_? '+ p_text[l_index])
              else display('  beyond');
*)
            l_comma_parenthesis:= f_string_extract_character(p_text, l_index);

            Inc(l_field_index);
          until l_comma_parenthesis= ')';

          display('< analyze_request');
        end; // analyze_request

      procedure execute_request;

        procedure do_exec_sql(p_request: String);
          var l_field_index: Integer;
              l_parameter_index: Integer;
          begin
            with l_c_ibsql do
            begin
              try
                display('exec');

                if DataBase.DefaultTransaction.InTransaction
                  then DataBase.DefaultTransaction.Commit;
                DataBase.DefaultTransaction.StartTransaction;

(*
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID,DesktopHtml,MobileSummary,MobileDetails) '
                    + ' VALUES (11, :DesktopHtml, :MobileSummary, :MobileDetails)');
*)
(*
                // -- OK
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ('+ IntToStr(100+ m_request_count_max)+ ')';
                // -- Not OK
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES ('+ IntToStr(100+ m_request_count_max)+ ')';
                Inc(m_request_count_max);
*)
                // -- mainly nok because of ParamCheck False
                // -- nok on Prepare
(*
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES (:MODULEID)';
                // -- OK
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ('+ IntToStr(100+ m_request_count_max)+ ')';
                // -- nok on Prepare
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ( :ModuleId )';
                p_request:= 'INSERT INTO Portal_HtmlText (MODULEID) '
                    + ' VALUES ( :x)';
                // -- ok
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES ( :MODULEID )';
*)
                Close;
                GenerateParamNames:= true;
                SQL.Text:= p_request;
                display('req '+ p_request+ '|');

                display('prepare');
                Prepare();

                display('fill params');
                Inc(m_request_count_max);

                l_parameter_index:= 0;
                with m_c_field_value_list do
                  for l_field_index:= 0 to f_field_value_count- 1 do
                    with f_c_field_value(l_field_index) do
                      if m_field_type in [ftBlob, ftMemo]
                        then begin
                            m_c_stream.Position:= 0;
                            display('> load_stream '+ Format('%7d pos %7d  sz %7d', [l_parameter_index, m_c_stream.Position, m_c_stream.Size]));
                            l_c_ibsql.Params[l_parameter_index].LoadFromStream(m_c_stream);
                            Inc(l_parameter_index);
                            display('< load_stream');
                          end;

                display('ExecQuery');
                ExecQuery;
                DataBase.DefaultTransaction.Commit;

                m_explain:= '  ok ';
                display(m_explain);
              except
                on e: Exception do
                  begin
                    m_explain:= '*** pb_'+ e.Message;
                    display(m_explain);
                  end;
              end; // try except
            end; // with l_c_ibsql
          end; // do_exec_sql

        var l_field_index: Integer;
            l_request: String;
            l_field_names, l_values: String;

        begin // execute_request
          display('> execute_request');
          // l_request:= 'INSERT INTO '+ UpperCase(l_table_name)+ ' (';
          l_request:= 'INSERT INTO '+ l_table_name+ ' (';

          with m_c_field_value_list do
          begin
            display_field_value_list;

            for l_field_index:= 0 to f_field_value_count- 1 do
              with f_c_field_value(l_field_index) do
              begin
                l_field_names:= l_field_names+ m_name;

                if l_field_index< f_field_value_count- 1
                  then l_field_names:= l_field_names+ ', ';
              end; // for l_field

            l_request:= l_request+ l_field_names+ ') VALUES (';

            l_values:= '';
            for l_field_index:= 0 to f_field_value_count- 1 do
              with f_c_field_value(l_field_index) do
              begin
                if m_field_type in [ftBlob, ftMemo]
                  then begin
                      l_values:= l_values+ ':'+ m_name;
                    end
                  else l_values:= l_values+ m_value;

                if l_field_index< f_field_value_count- 1
                  then l_values:= l_values+ ', ';
              end; // for l_field

            l_request:= l_request+ l_values+ ')';
          end; // with m_c_field_value_list

          display(l_request);

          if m_exec_sql
            then do_exec_sql(l_request);

          display('< execute_request');
        end; // execute_request

      begin // _insert_into_blob
        m_explain:= '???';

        create_ibsql_open_database;
        analyze_request(m_c_request.Text);
        execute_request;

        with l_c_ibsql do
          Free;
      end; // _insert_into_blob

    procedure c_insert_ib_blob.execute_script(p_full_script_file_name: String);
      var l_request, l_trimmed_request: String;
          l_index: Integer;
          l_table_name, l_previous_table_name: String;
      begin
        m_c_table_name_list.Free;
        m_c_table_name_list:= tStringList.Create;

        with c_ib_script.create_ib_script('script', p_full_script_file_name) do
        begin
          if f_initialized
            then begin
                display('scan');
                initialize_scanner;

                m_c_field_value_list:= Nil;
                l_previous_table_name:= '';

                repeat
                  l_request:= f_get_request;
                  if not (m_is_SET_TERM or m_is_COMMIT_WORK or m_is_SET_AUTODDL)
                    then begin
                        display('=== '+ m_terminator);

                        // l_request:= f_remove_all_quotes(l_request, '"');

                        // clear_display;
                        l_trimmed_request:= f_add_return_line_feed(l_request);
                        l_trimmed_request:= f_change_returns_in_spaces(l_trimmed_request);
                        l_trimmed_request:= Trim(f_remove_double_spaces(l_trimmed_request));
                        if l_trimmed_request<> ''
                          then begin
                              // Form1.trimmed_request_memo_.Lines.Add(l_trimmed_request);
                              // -- get the table name
                              l_index:= 1;
                              f_string_extract_identifier(l_trimmed_request, l_index);
                              f_string_extract_identifier(l_trimmed_request, l_index);
                              l_table_name:= f_string_extract_identifier(l_trimmed_request, l_index);
                              If m_c_table_name_list.IndexOf(l_table_name)< 0
                                then m_c_table_name_list.Add(l_table_name);

                              display(l_table_name);

                              if m_c_field_value_list= Nil
                                then begin
                                    m_c_field_value_list:= c_field_value_list.create_field_value_list('field_values');
                                    _create_field_value_list(l_table_name)
                                  end
                                else
                                  if l_table_name<> l_previous_table_name
                                    then begin
                                        m_c_field_value_list.Free;
                                        m_c_field_value_list:= c_field_value_list.create_field_value_list('field_values');
                                        _create_field_value_list(l_table_name);
                                      end
                                    else ;

                              l_previous_table_name:= l_table_name;

                              m_c_request.Text:= l_trimmed_request;
                              _insert_into_blob;
                            end;
                      end;
                until l_request= '';

                FreeAndNil(m_c_field_value_list);
              end; // could load

          Free;
        end; // with c_ib_script
      end; // execute_script

    Destructor c_insert_ib_blob.Destroy;
      begin
        m_c_request.Free;
        Inherited;
      end; // Destroy

    begin // u_c_insert_ib_blob
    end. // u_c_insert_ib_blob
