// 002 u_c_insert_ib_blob
// 16 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_insert_ib_blob;
  interface
    uses Classes, DbTables
        , IbSql
        , u_c_basic_object
        , u_types_constants
        , u_c_field_value_list
        ;

    type c_insert_ib_blob= class(c_basic_object)
                              m_table_name: String;
                              m_c_ib_sql_ref: tIbSql;
                              m_c_request: tStringList;
                              m_exec_sql: Boolean;
                              m_explain: String;

                              Constructor create_insert_ib_blob(p_name, p_table_name: String;
                                  p_c_ib_sql_ref: tIbSql);

                              procedure insert_into_blob(p_c_field_value_list: c_field_value_list);

                              Destructor Destroy; Override;
                            end; // c_insert_ib_blob

  implementation
    Uses SysUtils, Db, IbQuery, u_c_display
        , u_strings
        , u_ibx
        ;

    // -- c_insert_ib_blob

    Constructor c_insert_ib_blob.create_insert_ib_blob(p_name, p_table_name: String;
        p_c_ib_sql_ref: tIbSql);
      begin
        Inherited create_basic_object(p_name);

        m_table_name:= p_table_name;
        m_c_ib_sql_ref:= p_c_ib_sql_ref;

        m_c_request:= tStringList.Create;
      end; // create_insert_ib_blob

    var l_request_count: Integer= 5;
    
    procedure c_insert_ib_blob.insert_into_blob(p_c_field_value_list: c_field_value_list);
        // -- insert the usual field, use script for blob fields
      var l_table_name: String;
          // l_c_ib_query: tIbQuery;

      procedure analyze_request(p_text: String);
        var l_index, l_text_length: Integer;

        function f_extract_simple_field_value(p_field_type: tFieldType): String;
          begin
            case p_field_type of
              ftInteger : begin
                            Result:= f_string_extract_characters_in(p_text, l_index, k_digits);
                            // Result:= IntToStr(f_string_extract_integer(p_text, l_index));
                          end;
              ftString : f_string_extract_pascal_string(p_text, l_index);
              // -- todo: other types
            end; // f_string_extract_pascal_string(p_text, l_index)
          end; // f_extract_simple_field_value

        var l_insert, l_into: String;

            l_field_name: String;
            l_field_index: Integer;
            // l_field_type: tFieldType;
            l_field_index_of: Integer;

            l_parenthesis, l_comma_parenthesis: Char;
            l_values: String;
            l_string_value: String;
            // l_value_length: Integer;

        begin // analyze_request
          display('> analyze_request');

          l_index:= 1;
          l_text_length:= Length(p_text);
          l_insert:= f_string_extract_non_blank(p_text, l_index);
          l_into:= f_string_extract_non_blank(p_text, l_index);
          l_table_name:= f_string_extract_pascal_identifier(p_text, l_index);

          skip_blanks(p_text, l_index);
          l_parenthesis:= f_string_extract_character(p_text, l_index);

          l_field_index:= 0;
          repeat
            l_field_name:= f_string_extract_pascal_identifier(p_text, l_index);
            display('param '+ l_field_name);
            // l_c_field_value_list.Add(l_field_name);

            l_field_index_of:= p_c_field_value_list.f_index_of(l_field_name);
            if l_field_index_of>= 0
              then
                with p_c_field_value_list.f_c_field_value(l_field_index_of) do
                begin
                  // -- also add the CASE SENSITIVE name
                  m_name:= l_field_name;
                  m_field_index:= l_field_index;
                end;
(*
            if l_field_index in p_set_of_blob_field
              then l_field_type:= ftBlobFielde_non_blob_field
              else l_field_type:= e_binary_blob_field;
            l_c_field_value_list.f_c_add_field(l_field_name, l_field_type, '', Nil);
*)
            skip_blanks(p_text, l_index);
            l_comma_parenthesis:= f_string_extract_character(p_text, l_index);
            Inc(l_field_index);
          until l_comma_parenthesis= ')';

          l_values:= f_string_extract_pascal_identifier(p_text, l_index);

          skip_blanks(p_text, l_index);
          l_parenthesis:= f_string_extract_character(p_text, l_index);

          l_field_index:= 0;
          // display(p_text+ '|');
          repeat
            with p_c_field_value_list.f_c_field_value(l_field_index) do
              if m_field_type in [ftBlob, ftMemo]
                then begin
                    display('  blob');
                    // -- assumes string blob, with pascal '' escape
                    l_string_value:= f_string_extract_pascal_string(p_text, l_index);
                    // -- remove start and end quotes
                    l_string_value:= f_remove_quote(l_string_value, '''');
                    // display('    blob >'+ l_string_value+ '|');

                    // -- place in a stream
                    with tMemoryStream(m_c_stream) do
                    begin
                      Position:= 0;
                      if Length(l_string_value)> 0
                        then Write(l_string_value[1], Length(l_string_value));
                    end; // with tMemoryStream(m_c_stream)
(*
                    // -- "___,___" or  "__)__"
                    skip_blanks(p_text, l_index);
                    l_comma_parenthesis:= f_string_extract_character(p_text, l_index);
*)
                  end
                else begin
                    l_string_value:= f_extract_simple_field_value(m_field_type);
                    display('field '+ l_string_value+ '<');
                    m_value:= l_string_value;
                  end;
  (*
                    l_string_value:= f_string_extract_non_blank(p_text, l_index);
                    l_value_length:= Length(l_string_value);

                    if (l_value_length> 0) and (l_string_value[l_value_length] in [',', ')'])
                      then begin
                          // -- the delimiter is glued to the litteral
                          l_comma_parenthesis:= l_string_value[l_value_length];
                          System.Delete(l_string_value, l_value_length, 1);
                        end
                      else begin
                          // -- the delimiter is separated
                          // -- "___,___" or  "__)__"
                          skip_blanks(p_text, l_index);
                          if l_index<= l_text_length
                            then l_comma_parenthesis:= f_string_extract_character(p_text, l_index)
                            else display('beyond_len');;
                        end;
                    // l_c_field_value_list.f_c_field(l_field_index).m_value:= l_string_value;
                    display('  val '+ l_string_value);
                  end;

*)
            // -- "___,___" or  "__)__"
            skip_blanks(p_text, l_index);
(*
            if l_index<= l_text_length
              then display('  end_? '+ p_text[l_index])
              else display('  beyond');
*)
            l_comma_parenthesis:= f_string_extract_character(p_text, l_index);

            Inc(l_field_index);
          until l_comma_parenthesis= ')';

          display('< analyze_request');
        end; // analyze_request

      procedure execute_request;

        procedure do_exec_sql(p_request: String);
          var l_field_index: Integer;
              l_parameter_index: Integer;
          begin
            with m_c_ib_sql_ref do
            begin
              try
                display('exec');
                f_open_ib_database('open_ib_db', DataBase);
                if DataBase.DefaultTransaction.InTransaction
                  then DataBase.DefaultTransaction.Commit;
                DataBase.DefaultTransaction.StartTransaction;

(*
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID,DesktopHtml,MobileSummary,MobileDetails) '
                    + ' VALUES (11, :DesktopHtml, :MobileSummary, :MobileDetails)');
*)
(*
                // -- OK
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ('+ IntToStr(100+ l_request_count)+ ')';
                // -- Not OK
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES ('+ IntToStr(100+ l_request_count)+ ')';
                Inc(l_request_count);
*)
                // -- mainly nok because of ParamCheck False
                // -- nok on Prepare
(*
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES (:MODULEID)';
                // -- OK
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ('+ IntToStr(100+ l_request_count)+ ')';
                // -- nok on Prepare
                p_request:= 'INSERT INTO Portal_HtmlText (ModuleID) '
                    + ' VALUES ( :ModuleId )';
                p_request:= 'INSERT INTO Portal_HtmlText (MODULEID) '
                    + ' VALUES ( :x)';
                // -- ok
                p_request:= 'INSERT INTO PORTAL_HTMLTEXT (MODULEID) '
                    + ' VALUES ( :MODULEID )';
*)
                Close;
                GenerateParamNames:= true;
                SQL.Text:= p_request;
                display('req '+ p_request+ '|');
                // -- NOK INSERT INTO PORTAL_HTMLTEXT (MODULEID)  VALUES (:MODULEID)|

                display('prepare');
                Prepare();

                display('fill params');
                // m_c_ib_sql_ref.Params[0].AsInteger:= 1620 + l_request_count;
                Inc(l_request_count);

                l_parameter_index:= 0;
                with p_c_field_value_list do
                  for l_field_index:= 0 to f_field_value_count- 1 do
                    with p_c_field_value_list.f_c_field_value(l_field_index) do
                      if m_field_type in [ftBlob, ftMemo]
                        then begin
                            m_c_stream.Position:= 0;
                            display('> load_stream '+ Format('%7d pos %7d  sz %7d', [l_parameter_index, m_c_stream.Position, m_c_stream.Size]));
                            m_c_ib_sql_ref.Params[l_parameter_index].LoadFromStream(m_c_stream);
                            Inc(l_parameter_index);
                            display('< load_stream');
                          end;

                display('ExecQuery');
                ExecQuery;
                DataBase.DefaultTransaction.Commit;

                m_explain:= '  ok ';
                display(m_explain);
              except
                on e: Exception do
                  begin
                    m_explain:= '*** pb_'+ e.Message;
                    display(m_explain);
                  end;
              end; // try except
            end; // with m_c_ib_sql_ref
          end; // do_exec_sql

        var l_field_index: Integer;
            l_request: String;
            l_field_names, l_values: String;

        begin // execute_request
          display('> execute_request');
          // l_request:= 'INSERT INTO '+ UpperCase(l_table_name)+ ' (';
          l_request:= 'INSERT INTO '+ l_table_name+ ' (';

          with p_c_field_value_list do
          begin
            display_field_value_list;

            for l_field_index:= 0 to f_field_value_count- 1 do
              with p_c_field_value_list.f_c_field_value(l_field_index) do
              begin
                l_field_names:= l_field_names+ m_name;

                if l_field_index< f_field_value_count- 1
                  then l_field_names:= l_field_names+ ', ';
              end; // for l_field

            l_request:= l_request+ l_field_names+ ') VALUES (';

            l_values:= '';
            for l_field_index:= 0 to f_field_value_count- 1 do
              with p_c_field_value_list.f_c_field_value(l_field_index) do
              begin
                if m_field_type in [ftBlob, ftMemo]
                  then begin
                      l_values:= l_values+ ':'+ m_name;
                      // m_c_ib_sql_ref.ParamByName(m_name).s[l_field_index].LoadFromStream(m_c_stream, ftBlob);
                      // m_c_ib_sql_ref.ParamByName(m_name).LoadFromStream(m_c_stream);
                    end
                  else l_values:= l_values+ m_value;

                if l_field_index< f_field_value_count- 1
                  then l_values:= l_values+ ', ';
              end; // for l_field

            l_request:= l_request+ l_values+ ')';
          end; // with p_c_field_value_list

          display(l_request);

          if m_exec_sql
            then do_exec_sql(l_request);

          display('< execute_request');
        end; // execute_request

      begin // insert_into_blob
        m_explain:= '???';

        with m_c_ib_sql_ref do
        begin
          // Sql.Assign(m_c_request);

          // -- must open to initialize the params ?
          if m_exec_sql
            then f_open_ib_database('open_ib_db_2', m_c_ib_sql_ref.DataBase);
          analyze_request(m_c_request.Text);

          execute_request;
        end; // with m_c_ib_sql_ref
      end; // insert_into_blob

    Destructor c_insert_ib_blob.Destroy;
      begin
        m_c_request.Free;
        Inherited;
      end; // Destroy

    begin // u_c_insert_ib_blob
    end. // u_c_insert_ib_blob
